////////////////////////////////////////////////////////////////////////////
//
// nrMaterial.cpp
//
// A class for materials.
//
// Nate Robins, January 2002.
//
////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////

#include "nrMaterial.h"

#include "nrMaterialLibrary.h"
#include "nrParser.h"


////////////////////////////////////////////////////////////////////////////
// Public
////////////////////////////////////////////////////////////////////////////

nrMaterial::nrMaterial( const nrColor& ambient, const nrColor& diffuse, const nrColor& specular )
{
    m_Name = 0;
    m_Ambient = ambient;
    m_Diffuse = diffuse;
    m_Specular = specular;
}

////////////////////////////////////////////////////////////////////////////

nrMaterial::~nrMaterial( void )
{
    delete [] m_Name;
}

////////////////////////////////////////////////////////////////////////////

void nrMaterial::SetName( const char* name )
{
    delete [] m_Name;
    m_Name = 0;
    
    if ( name )
    {
        m_Name = new char[ strlen( name ) + 1 ];
        strcpy( m_Name, name );
    }
}

////////////////////////////////////////////////////////////////////////////

const char* nrMaterial::GetName( void )
{
    return m_Name;
}

////////////////////////////////////////////////////////////////////////////

nrMaterial* nrMaterial::ParseColor( nrParser& parser )
{
    nrColor color;
    
    parser.ReadToken( nrParser::TOKEN_LIST_BEGIN );
    color.r = parser.ReadFloat();
    color.g = parser.ReadFloat();
    color.b = parser.ReadFloat();
    parser.ReadToken( nrParser::TOKEN_LIST_END );
    
    if ( parser.Error() )
    {
        return 0;
    }
    
    nrMaterial* material = new nrMaterial( color, color );
    
    nrMaterialLibrary::Add( material );
    
    return material;
}

////////////////////////////////////////////////////////////////////////////

nrMaterial* nrMaterial::Parse( nrParser& parser )
{
    const char* name = 0;
    char* new_name = 0;
    
    nrColor ambient = nrColor( 0, 0, 0 );
    nrColor diffuse = nrColor( 1, 1, 1 );
    nrColor specular = nrColor( 0, 0, 0 );
    
    if ( parser.NextToken() == nrParser::TOKEN_STRING )
    {
        name = parser.ReadString();
    }
    
    if ( name )
    {
        nrMaterial* m = nrMaterialLibrary::Find( name );
        if ( m )
        {
            return m;
        }
        
        new_name = new char[ strlen( name ) + 1 ];
        strcpy( new_name, name );
    }
    
    parser.ReadToken( nrParser::TOKEN_AGGREGATE_BEGIN );
    
    while ( parser.NextToken() == nrParser::TOKEN_KEY )
    {
        const char* key = parser.ReadKey();
        
        if ( parser.KeyMatches( key, "ambient" ) )
        {
            parser.ReadToken( nrParser::TOKEN_LIST_BEGIN );
            ambient.r = parser.ReadFloat();
            ambient.g = parser.ReadFloat();
            ambient.b = parser.ReadFloat();
            parser.ReadToken( nrParser::TOKEN_LIST_END );
        }
        else if ( parser.KeyMatches( key, "diffuse" ) )
        {
            parser.ReadToken( nrParser::TOKEN_LIST_BEGIN );
            diffuse.r = parser.ReadFloat();
            diffuse.g = parser.ReadFloat();
            diffuse.b = parser.ReadFloat();
            parser.ReadToken( nrParser::TOKEN_LIST_END );
        }
        else if ( parser.KeyMatches( key, "specular" ) )
        {
            parser.ReadToken( nrParser::TOKEN_LIST_BEGIN );
            specular.r = parser.ReadFloat();
            specular.g = parser.ReadFloat();
            specular.b = parser.ReadFloat();
            parser.ReadToken( nrParser::TOKEN_LIST_END );
        }
        else
        {
            parser.ParseError( "unknown key." );
        }
    }
    
    parser.ReadToken( nrParser::TOKEN_AGGREGATE_END );
    
    if ( parser.Error() )
    {
        return 0;
    }
    
    nrMaterial* material = new nrMaterial( ambient, diffuse, specular );
    material->SetName( new_name );
    
    delete [] new_name;
    
    nrMaterialLibrary::Add( material );
    
    return material;
}

////////////////////////////////////////////////////////////////////////////
