////////////////////////////////////////////////////////////////////////////
//
// nrMaterialLibrary.h
//
// A class for materials.
//
// Nate Robins, January 2002.
//
////////////////////////////////////////////////////////////////////////////

#ifndef NRMATERIALLIBRARY_H
#define NRMATERIALLIBRARY_H


////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////

#include "nrArray.h"
#include "nrMaterial.h"


////////////////////////////////////////////////////////////////////////////
// Classes
////////////////////////////////////////////////////////////////////////////

class nrMaterial;

////////////////////////////////////////////////////////////////////////////

class nrMaterialLibrary
{
private:

    nrMaterialLibrary( void );
    ~nrMaterialLibrary( void );
    
public:
    
    // Add a material to the library.
    //
    // If a material exists of the same name, the new one is not added and a
    // warning is issued.
    static void Add( nrMaterial* material );
    
    // Return a material with the name specified.
    //
    // If the material couldn't be found in the library, 0 is returned.
    static nrMaterial* Find( const char* name );
    
    // Initialize the materials library.
    static void Initialize( void );
    
    // Terminate the materials library (delete all materials).
    static void Terminate( void );
    
private:
    
    static nrArray< nrMaterial* > m_Materials;
};

////////////////////////////////////////////////////////////////////////////

#endif  // NRMATERIALLIBRARY_H
