////////////////////////////////////////////////////////////////////////////
//
// nrPencil.cpp
//
// A class for writing (a pencil).
//
// Nate Robins, March 2001.
//
////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////

#include "nrPencil.h"

#include <assert.h>
#include <crtdbg.h>
#include <stdio.h>


////////////////////////////////////////////////////////////////////////////
// Globals
////////////////////////////////////////////////////////////////////////////

nrPencil g_Pencil;


////////////////////////////////////////////////////////////////////////////
// Public
////////////////////////////////////////////////////////////////////////////

nrPencil::nrPencil( void )
{
    m_Buffer = NULL;
    m_BufferSize = 64;
    
    Sharpen();
}

////////////////////////////////////////////////////////////////////////////

nrPencil::~nrPencil( void )
{
    delete [] m_Buffer;
}

////////////////////////////////////////////////////////////////////////////

const char* nrPencil::Write( const char* format, ... )
{
    va_list args;
    va_start( args, format );
    const char* string = Compose( format, args );
    va_end( args );
    
    return string;
}

////////////////////////////////////////////////////////////////////////////

const char* nrPencil::Compose( const char* format, va_list args )
{
    int num_chars;
    
    do
    {
        num_chars = _vsnprintf( m_Buffer, m_BufferSize, format, args );
        
        if ( num_chars == -1 )
        {
            Sharpen();
        }
    }
    while ( num_chars == -1 );
    
    return m_Buffer;
}

////////////////////////////////////////////////////////////////////////////
// Private
////////////////////////////////////////////////////////////////////////////

void nrPencil::Sharpen( void )
{
    delete [] m_Buffer;
    
    m_BufferSize *= 2;
    
    m_Buffer = new char[ m_BufferSize ];
    
    assert( m_Buffer );
    
    // Sanity check.
#if defined ( _DEBUG )
    {
        static int sharpens = 0;
        if ( ++sharpens > 6 )
        {
            _RPT0( _CRT_WARN, "nrPencil::Sharpen() : The pencil was Sharpen()'d a bunch of times (buffer size is probably getting too big)!" );
        }
        
        _RPT2( _CRT_WARN, "nrPencil<0x%08X>::Sharpen() : %d bytes\n", this, m_BufferSize );
    }
#endif
}

////////////////////////////////////////////////////////////////////////////
